# import the needed modules
try:
#  import sys
  import numpy
  import pyfits
  import wx
  import matplotlib
  from reflex import parseSof,RecipeParam
  from reflex_interactive_app import PipelineInteractiveApp
  from pipeline_product import PipelineProduct
  from pipeline_display import SpectrumDisplay, ImageDisplay, ScatterDisplay

#  matplotlib.use('WXAgg')
  import_sucess = 'true'

#NOTE for developers: 
# -If you want to modify the current script to cope
#  with different parameters, this is the function to modify:
#  setInteractiveParameters()
# -If you want to modify the current script to read different data from
#  the input FITS, these are the functions (class DataPlotterManager) to modify:
#  readFitsData()    to indicate what are the FITS input to plot
#  plotSpectrum()    to control 1D image product layout
#  plotImage()       to control 2D image product layout
#  plotTable()       to control 1D table product layout
#  oplotTable()      to control 1D table product overplots
#  getSN()           to access 1D spectrum S/N FITS header info
#  setInsMode()      to access ins mode FITS header info
#  setObjType()      to access obj type FITS header info
#  setRecId()        to set rec id (used in parameter prefix)
#  getArmId()        to get arm id FITS header info
#
# -If you want to modify the current script to modify the plots (using the same
#  data),  this is the function to modify:
#  plotProductsGraphics()          (from class DataPlotterManager)
# -If you want to modify the text that appears in the "Help" button,
#  this is the function to modify:
#  setWindowHelp()
# -If you want to modify the tittle of the window, modify this function:
#  setWindowTittle()


  #This class deals with the specific details of data reading and final plotting.
  class DataPlotterManager:

    #1D spectrum plots
    def plotSpectrum(self,data,errs,subplot,title,tooltip):
        data.readSpectrum(0)
        errs.readSpectrum(1)
        spectrum = SpectrumDisplay()
        spectrum.setLabels('Wavelength [nm]','Total Flux ['+data.bunit+']')
        spectrum.display(subplot,title,tooltip,data.wave,data.flux,errs.flux) 

    #2D image display
    def plotImage(self,obj,bpm,subplot,title,tooltip):

          obj.readImage(fits_extension=0)
          obj.read2DLinearWCS(fits_extension=0)
          img_obj = ImageDisplay()

          bpm.readImage(fits_extension=2)
          img_bpm = ImageDisplay()

	  img_obj.setLabels('Wavelength [nm]','Position Along Slit [arcsec]')
          y_size, x_size = obj.image.shape

          img_obj.setXLinearWCSAxis(obj.crval1,obj.cdelt1,obj.crpix1)
          img_obj.setYLinearWCSAxis(obj.crval2,obj.cdelt2,obj.crpix2)

	  img_obj.setLimits((0,x_size),(0,y_size))
          img_obj.display(subplot, title, tooltip,obj.image,bpm.image)

    #1D spectrum table plots
    def plotTable(self,tab,Xcol,Ycol,Xlab,Ylab,subplot,title,tooltip):

        tab.readTableXYColumns(fits_extension=1,xcolname=Xcol, ycolname=Ycol)
        spectrum = SpectrumDisplay()
        spectrum.setLabels(Xlab,Ylab)
        spectrum.display(subplot,title,tooltip,tab.x_column,tab.y_column) 

    #1D spectrum table over-plots
    def oplotTable(self,tab,Xcol,Ycol,Xlab,Ylab,subplot,color):

        tab.readTableXYColumns(fits_extension=1,xcolname=Xcol, ycolname=Ycol)
        spectrum = SpectrumDisplay()
        spectrum.overplot(subplot,tab.x_column,tab.y_column,color)

    #Get S/N info (Xshooter and arm specific)
    def getSN(self, hdulist):
      self.SN1 = hdulist[0].header['ESO QC FLUX1 SN']
      self.SN2 = hdulist[0].header['ESO QC FLUX2 SN']
      if self.arm_uvb_found  == True :
        self.RN1 = "450-470 nm"
        self.RN2 = "510-530 nm"
      if self.arm_vis_found  == True :
        self.SN3 = hdulist[0].header['ESO QC FLUX3 SN']
        self.RN1 = "672-680 nm"
        self.RN2 = "745-756 nm"
        self.RN3 = "992-999 nm"
      if self.arm_nir_found  == True :
        self.RN1 = "1514-1548 nm"
        self.RN2 = "2214-2243 nm"


    # This function will read all the columns, images and whatever is needed
    # from the products. The variables , self.plot_x, self.plot_y, etc...
    # are used later in function plotProductsGraphics().
    # Add/delete these variables as you need (only that plotProductsGraphics()
    # has to use the same names).
    # You can also create some additional variables (like statistics) after
    # reading the files.
    # If you use a control variable (self.xxx_found), you can modify 
    # later on the layout of the plotting window based on the presence of 
    # given input files. 
    # sof contains all the set of frames
    def readFitsData(self, fitsFiles):
      #Control variable to check if the interesting files where at the input
      prefix = "SCI_SLIT_"
      self.spectrum1D_found   = False
      self.flux_spectrum1D_found   = False
      self.lineguess_found = False
      self.spectrum2D_found      = False
      self.response_found      = False
      self.mresponse_found      = False
      self.sky_spectrum2D_found      = False
      self.sky2D_found   = False
      self.obj_sky2D_found   = False
      self.bpm_found   = False
      self.arm_uvb_found      = False
      self.arm_vis_found      = False
      self.arm_nir_found      = False
      self.nima = 0
      
      #Read all the products
      hdus = dict()
      for frame in fitsFiles:
        if frame == '' :
          continue

        hdulist = frame.hdulist()
        category = frame.category
        hdus[category] = hdulist

      prefixes = [k.split('_')[0] for k in hdus.keys()]
      if "SCI" in prefixes:
        prefix = "SCI_SLIT_"
      elif "TELL" in prefixes:
        prefix = "TELL_SLIT_"
      elif "FLUX" in prefixes:
        prefix = "FLUX_SLIT_"
 
      arm=self.seq_arm

      if arm == "UVB":
 	 self.arm_uvb_found = True
      if arm == "VIS":
 	 self.arm_vis_found = True
      if arm == "NIR":
 	 self.arm_nir_found = True

# For any arm search a list of input frames
      if hdus.has_key(prefix+"MERGE1D_"+arm)  :
        self.spectrum1D_found = True
        hdulist = hdus[prefix+"MERGE1D_"+arm]
        self.spectrum1D = PipelineProduct(hdulist)
        self.error1D = PipelineProduct(hdulist)
        self.getSN(hdulist)
        self.nima += 1

      if hdus.has_key(prefix+"FLUX_MERGE1D_"+arm)  :
        self.flux_spectrum1D_found = True
        hdulist = hdus[prefix+"FLUX_MERGE1D_"+arm]
        self.flux_spectrum1D = PipelineProduct(hdulist)
        self.flux_error1D = PipelineProduct(hdulist)
        self.nima += 1
	

      if hdus.has_key(prefix+"MERGE2D_"+arm)  :
        self.spectrum2D_found = True
        hdulist = hdus[prefix+"MERGE2D_"+arm]
        self.bpm = PipelineProduct(hdulist)
        self.spectrum2D = PipelineProduct(hdulist)
        self.nima += 1

      if hdus.has_key("SKY_SLIT_MERGE2D_"+arm) and self.rec_id[16:19] != "nod":
        self.sky_spectrum2D_found = True
        hdulist = hdus["SKY_SLIT_MERGE2D_"+arm]
        self.sky_spectrum2D = PipelineProduct(hdulist)
        self.nima += 1

      if hdus.has_key(prefix+"SKY_"+arm)  :
        self.sky2D_found = True
        hdulist = hdus[prefix+"SKY_"+arm]
        self.sky2D = PipelineProduct(hdulist)

      if hdus.has_key(prefix+"SUB_SKY_"+arm) :
        self.obj_sky2D_found = True
        hdulist = hdus[prefix+"SUB_SKY_"+arm]
        self.obj_sky2D = PipelineProduct(hdulist)

      if self.rec_id[0:10] == "xsh_respon":

         if hdus.has_key("RESPONSE_MERGE1D_SLIT_"+arm)  : 
            self.response_found = True
            hdulist = hdus["RESPONSE_MERGE1D_SLIT_"+arm]
            self.response = PipelineProduct(hdulist)
            self.nima += 1

         if hdus.has_key("MRESPONSE_MERGE1D_SLIT_"+arm)  : 
            self.mresponse_found = True
            hdulist = hdus["MRESPONSE_MERGE1D_SLIT_"+arm]
            self.mresponse = PipelineProduct(hdulist)

    #get ins mode (Slit/IFU and Stare/Offset/Nodding)
    def setInsMode(self, rec_id):
      #Recipe ID variable to properly define params
      self.rec_id = rec_id

      if self.rec_id[16:21] == "stare" :
         self.ins_mode = "Slit-Stare"
      elif self.rec_id[16:22] == "offset" :
         self.ins_mode = "Slit-Offset"

      elif self.rec_id[16:19] == "nod" :
         self.ins_mode = "Slit-Nodding"
      elif self.rec_id == "xsh_scired_ifu_stare" :
         self.ins_mode = "ifu-stare"
      elif self.rec_id == "xsh_scired_ifu_offset" :
         self.ins_mode = "ifu-offset"

    #Set Object type (Object/Std star) to display proper info
    def setObjType(self, rec_id):
      #Recipe ID variable to properly define params
      self.rec_id = rec_id

      if self.rec_id[0:10] == "xsh_scired":
         self.obj_type = "Object"
      
      if self.rec_id[0:10] == "xsh_respon":
         self.obj_type = "STD"



    #Set rec id (to have proper recipe parameters prefix)
    def setRecId(self, rec_id):
      #Recipe ID variable to properly define params
      self.rec_id = rec_id
      self.setInsMode(rec_id)
      self.setObjType(rec_id)

    #Get arm setting
    def getArmId(self, sof):
      #Recipe ID variable to properly define params
      self.rec_id = "xsh_respon_slit_stare"
      self.seq_arm = "UVB"
      nf = 0

      hdus = dict()
      files, dataset = parseSof(sof)
      for f in files:
        frame=f.name
        if frame == '' :
          continue
        else :
           nf += 1
           hdulist = pyfits.open(frame)
           rec_id_list = hdulist[0].header['ESO PRO REC1 ID']
           arm_list = hdulist[0].header['ESO SEQ ARM']
      if nf != 0 :
         self.rec_id = rec_id_list[0:]
         self.seq_arm = arm_list[0:]

      if self.seq_arm == "UVB":
         self.arm_uvb_found = True

      if self.seq_arm == "VIS":
         self.arm_uvb_found = True

      if self.seq_arm == "NIR":
         self.arm_uvb_found = True

      if self.rec_id[16:21] == "stare" :
         self.ins_mode = "Slit-Stare"
      elif self.rec_id[16:22] == "offset" :
         self.ins_mode = "Slit-Offset"

      elif self.rec_id[16:19] == "nod" :
         self.ins_mode = "Slit-Nodding"
      elif self.rec_id == "xsh_scired_ifu_stare" :
         self.ins_mode = "ifu-stare"
      elif self.rec_id == "xsh_scired_ifu_offset" :
         self.ins_mode = "ifu-offset"

      if self.rec_id[0:10] == "xsh_scired":
         self.obj_type = "Object"
      
      if self.rec_id[0:10] == "xsh_respon":
         self.obj_type = "STD"



    # This function creates all the subplots. It is responsible for the plotting 
    # layouts. 
    # There can different layouts, depending on the availability of data
    # Note that subplot(I,J,K) means the Kth plot in a IxJ grid 
    # Note also that the last one is actually a box with text, no graphs.
    def addSubplots(self, figure):
      nrows=self.nima
      row=1;

      if self.spectrum1D_found == True and self.spectrum2D_found == True:
        self.subplot_spectrum  = figure.add_subplot(nrows,1,row)
        row +=1

        if self.flux_spectrum1D_found == True :
           self.subplot_flux_spectrum = figure.add_subplot(nrows,1,row)
           row +=1

        if self.response_found == True:
           self.subplot_response    = figure.add_subplot(nrows,1,row)
           row +=1

        if self.spectrum2D_found == True :
           self.subplot_image_obj = figure.add_subplot(nrows,1,row)
           row +=1

        if self.rec_id[17:19] != "nod" :

          if self.sky_spectrum2D_found == True :
             self.subplot_image_sky = figure.add_subplot(nrows,1,row)
             row +=1

      else : 
        self.subtext_nodata    = figure.add_subplot(1,1,1)
        print "found no spectrum data"


          
    # This is the function that makes the plots.
    # Add new plots or delete them using the given scheme.
    # The data has been already stored in self.plot_x, self.plot_xdif, etc ...
    # It is mandatory to add a tooltip variable to each subplot.
    # One might be tempted to merge addSubplots() and plotProductsGraphics().
    # There is a reason not to do it: addSubplots() is called only once at
    # startup, while plotProductsGraphics() is called always there is a resize.
    def plotProductsGraphics(self, figure, canvas):
      if self.spectrum1D_found == True and self.spectrum2D_found == True: 
        #and self.lineguess_found == True 
        title_pref = 'Linear-extracted and Merged Spectrum.'	      
        title_flux_spectrum = 'Linear Extracted, Merged, and Flux Calibrated Spectrum.'
        tooltip_spectrum ="""Plot of the linear-extracted and merged spectrum of the object (blue line) as total flux (ADU) versus wavelength (nm). The +-1 sigma uncertainties are plotted as upper and lower red curves. Note that this spectrum is not flux calibrated."""

        tooltip_flux_spectrum ="""Plot of the linear-extracted, merged, and flux calibrated spectrum of the object (blue line) as total flux (10^-16 erg/s/cm^2/A) versus wavelength (nm). The +-1 sigma uncertainties are plotted as upper and lower red curves."""

	if self.arm_uvb_found == True:
      	  title_spectrum   = title_pref+' \n SN: %#.2f (%#s); %#.2f (%#s) '% (self.SN1,self.RN1,self.SN2,self.RN2)

	if self.arm_vis_found == True:
      	  title_spectrum   = title_pref+' \n SN: %#.2f (%#s); %#.2f (%#s); %#.2f (%#s) '% (self.SN1,self.RN1,self.SN2,self.RN2,self.SN3,self.RN3)

	if self.arm_nir_found == True:
      	  title_spectrum   = title_pref+' \n SN: %#.2f (%#s); %#.2f (%#s); '% (self.SN1,self.RN1,self.SN2,self.RN2)
	   
	   
        #Spectrum plot

        self.plotSpectrum(self.spectrum1D,self.error1D,self.subplot_spectrum,title_spectrum,tooltip_spectrum)

        #Flux calibrated Spectrum plot

        if self.flux_spectrum1D_found == True:

           self.plotSpectrum(self.flux_spectrum1D,self.flux_error1D,self.subplot_flux_spectrum,title_flux_spectrum,tooltip_flux_spectrum)


        #Image
        title_image_obj   = '2D Merged ' + self.obj_type + ' Spectrum' 
        tooltip_image_obj ="""Image of the 2D merged spectrum of the object in ADU. Bad pixels are flagged with a red hue. Note that this spectrum is not flux calibrated."""


        title_image_sky   = '2D Merged Sky Spectrum ' 
        tooltip_image_sky ="""Image of the 2D merged spectrum of the sky in ADU. Bad pixels are flagged with a red hue. Note that this spectrum is not flux calibrated."""
        #Object merged 2D
        if self.spectrum2D_found  == True:
           self.plotImage(self.spectrum2D,self.bpm,self.subplot_image_obj,title_image_obj,tooltip_image_obj)


        #sky merged 2D
        if self.sky_spectrum2D_found  == True and self.rec_id != "xsh_respon_slit_nod" and self.rec_id != "xsh_scired_slit_nod" :

           self.plotImage(self.sky_spectrum2D,self.bpm,self.subplot_image_sky,title_image_sky,tooltip_image_sky)


        #Scatter plot
        if self.response_found == True :
           title_response   = 'Instrument response curve' 
           tooltip_response ="""Plot of the instrument response curve [blue],
        compared with the one of the master response curve [green], if the
        user provides it in input. Units are erg/e-/cm2"""
           self.plotTable(self.response,'LAMBDA','RESPONSE','Wavelength [nm]','Response',self.subplot_response,title_response,tooltip_response)

        if self.mresponse_found == True :
           self.oplotTable(self.mresponse,'LAMBDA','RESPONSE','Wavelength [nm]','Response',self.subplot_response,'green')
  
      else :
        #Data not found info
        self.subtext_nodata.set_axis_off()
        self.text_nodata = """Merged spectrum
       (PRO.CATG=PREF_MERGE1D_ARM,PREF_MERGE2D_ARM, 
       where PREF=FLUX_SLIT, TELL_SLIT or SCI_SLIT, 
       ARM=UVB,VIS or NIR) not found in the products. 
       This may be due to a recipe failure. 
      Check your input parameter values, 
      correct possibly typos and
      press 'Re-run recipe' button."""
        self.subtext_nodata.text(0.1, 0.6, self.text_nodata, color='#11557c', 
                                 fontsize=18, ha='left', va='center', alpha=1.0)
        self.subtext_nodata.tooltip='Merged spectrum not found in the products'
        print "found no spectrum data"
 
  
    # This function specifies which are the parameters that should be presented
    # in the window to be edited.
    # Note that the parameter has to be also in the in_sop port (otherwise it 
    # won't appear in the window) 
    # The descriptions are used to show a tooltip. They should match one to one
    # with the parameter list 
    # Note also that parameters have to be prefixed by the 'recipe name:'

    def setGeneralParameters(self,paramList,rec_id):
      group="general"
      help1="""Integer representation of the bits to be considered bad when 
            decoding the bad pixel mask pixel values."""
      paramList.append(RecipeParam(rec_id,"decode-bp",group,help1))
      help2="""pre-overscan correction.0: no correction1: mean overscan 
               correction2: mean prescan 
               correction3: (mean pre+mean overscan)/2 correction."""
      paramList.append(RecipeParam(rec_id,"pre-overscan-corr",group,help2))
      help3="""Method for combining overlapping orders (0 = WEIGHTED with the 
             errors, 1 = MEAN)."""
      paramList.append(RecipeParam(rec_id,"mergeord-method",group,help3))
      help4="""TRUE if we do the flatfielding."""
      paramList.append(RecipeParam(rec_id,"do-flatfield",group,help4))

      help5="""if TRUE recompute (wave and slit) maps from the dispersion 
          solution. Use FALSE only if you are sure to have wave and slit maps 
          better than what this recipe may compute.""" 

      if rec_id[16:21] == "stare" :
         help5=help5+"If sky-subtract is set to TRUE this must be set to TRUE."

      help6="""if TRUE compute object position trace via Gaussian fit."""
      help7="""Method used for extraction (LOCALIZATION, NOD)."""
      help8="""if TRUE a 2D sky frame, a 2D rectified, a 2D merged
                          sky are generated."""
      help9="""Lower Slit Limit (localize and extract."""
      help10="""Upper Slit Limit (localize and extract."""
      if rec_id == "xsh_scired_slit_stare" or rec_id == "xsh_respon_slit_stare":
      
        paramList.append(RecipeParam(rec_id,"compute-map",group,help5))
        paramList.append(RecipeParam(rec_id,"trace-obj",group,help6))
        paramList.append(RecipeParam(rec_id,"extract-method",group,help7))
      if rec_id == "xsh_scired_slit_offset" or rec_id == "xsh_respon_slit_offset":
        paramList.append(RecipeParam(rec_id,"gen-sky",group,help8))

      if rec_id == "xsh_scired_slit_nod" or rec_id == "xsh_respon_slit_nod":
        paramList.append(RecipeParam(rec_id,"extract-method",group,help7))
        paramList.append(RecipeParam(rec_id,"max-slit",group,help9))
        paramList.append(RecipeParam(rec_id,"min-slit",group,help10))


    def setStackParameters(self,paramList,rec_id):
      group="stack"
      help1="""Method used to build master frame. <median | mean>"""
      help2="""Kappa used to clip low level values, when method is set to 
            'mean'."""
      help3="""Kappa used to clip high level values, when method is set to 
            'mean'."""
      help4="""Number of kappa sigma iterations, when method is set to 
            'mean'."""

      paramList.append(RecipeParam(rec_id,"stack-method",group,help1))
      paramList.append(RecipeParam(rec_id,"klow",group,help2))
      paramList.append(RecipeParam(rec_id,"khigh",group,help3))
      paramList.append(RecipeParam(rec_id,"niter",group,help4))

    def setCrhClipParameters(self,paramList,rec_id):
      group="crh-clip"
      help1="""Kappa value in sigma clipping during CRH rejection
                          using multiple frames."""
      help2="""Number of iterations in sigma clipping during CRH
                          rejection using multiple frames."""
      help3="""Minimal ratio of points accepted / total in sigma
               clipping during CRH rejection using multiple frames."""


      paramList.append(RecipeParam(rec_id,"crh-clip-kappa",group,help1))
      paramList.append(RecipeParam(rec_id,"crh-clip-niter",group,help2))
      paramList.append(RecipeParam(rec_id,"crh-clip-frac",group,help3))

    def setCrhSingleParameters(self,paramList,rec_id):
      group="removecrhsingle"
      help1="""Max fraction of bad pixels allowed."""
      help2="""Poisson fluctuation threshold to flag CRHs (see
               van Dokkum, PASP,113,2001,p1420-27)."""
      help3="""Minimum contrast between the Laplacian image and the
               fine structure image that a point must have to be
               flagged as CRH. (see van Dokkum, PASP,113,2001,p1420-27)."""
      help4="""Max number of iterations."""
      paramList.append(RecipeParam(rec_id,"removecrhsingle-frac-max",group,help1))
      paramList.append(RecipeParam(rec_id,"removecrhsingle-sigmalim",group,help2))
      paramList.append(RecipeParam(rec_id,"removecrhsingle-flim",group,help3))
      paramList.append(RecipeParam(rec_id,"removecrhsingle-niter",group,help4))

    def setBackgroundParameters(self,paramList,rec_id):
      group="background"
      help1="""number of points of the grid in y direction."""
      help2="""half size of the subwindow in x direction (2 for
                          UVB,VIS, 1 for NIR) [pixel units]."""
      help3="""half size of the subwindow in y direction [pixel units].""" 
      help4="""Minimum fraction of points used to calculate
                                median to the grid."""
      help5="""maximum fraction of points rejected from the grid."""
      help6="""method adopted for background estimation:. <median |
                          minimum | poly | no>""" 
      help7="""Smoothing radius' half size in x direction [bin
                          units]. If negative no smoothing is performed."""
      help8="""Smoothing radius' half size in y direction [bin
                          units]. If negative no smoothing is performed."""
      help9="""Poly mode fit deg along X."""
      help10="""Poly mode fit deg along Y."""
      help11="""Poly mode kappa value of kappa-sigma-clip outliers removal."""

      paramList.append(RecipeParam(rec_id,"background-nb-y",group,help1))
      paramList.append(RecipeParam(rec_id,"background-radius-x",group,help2))
      paramList.append(RecipeParam(rec_id,"background-radius-y",group,help3))
      paramList.append(RecipeParam(rec_id,"background-min-grid-frac",group,help4))
      paramList.append(RecipeParam(rec_id,"background-grid-frac",group,help5))
      paramList.append(RecipeParam(rec_id,"background-method",group,help6))
      paramList.append(RecipeParam(rec_id,"background-smooth-x",group,help7))
      paramList.append(RecipeParam(rec_id,"background-smooth-y",group,help8))
      paramList.append(RecipeParam(rec_id,"background-poly-deg-x",group,help9))
      paramList.append(RecipeParam(rec_id,"background-poly-deg-y",group,help10))
      paramList.append(RecipeParam(rec_id,"background-poly-kappa",group,help11))


    def setLocalizeParameters(self,paramList,rec_id):
      group="localize"
      help1="""Localization method (MANUAL, MAXIMUM, GAUSSIAN) used
               to detect the object centroid and height on the slit."""
      help2="""Number of chunks in the full spectrum to localize the object."""
      help3="""Threshold relative to the peak intensity below which the
               edges of the object are detected for MAXIMUM localization."""
      help4="""Degree in lambda in the localization polynomial expression 
               slit=f(lambda), used only for MAXIMUM and GAUSSIAN."""
      help5="""Object position on the slit for MANUAL localization [arcsec]."""
      help6="""Object half height on the slit for MANUAL
               localization [arcsec]."""
      help7="""Kappa value for sigma clipping in the localization
               polynomial fit."""
      help8="""Number of iterations for sigma clipping in the
                          localization polynomial fit."""
      help9="""TRUE if we want to mask sky lines using SKY_LINE_LIST file."""
      help10="""Step (arcsec) between A and B images in nodding mode."""

      paramList.append(RecipeParam(rec_id,"localize-method",group,help1))
      paramList.append(RecipeParam(rec_id,"localize-chunk-nb",group,help2))
      paramList.append(RecipeParam(rec_id,"localize-thresh",group,help3))
      paramList.append(RecipeParam(rec_id,"localize-deg-lambda",group,help4))
      paramList.append(RecipeParam(rec_id,"localize-slit-position",group,help5))
      paramList.append(RecipeParam(rec_id,"localize-slit-hheight",group,help6))
      paramList.append(RecipeParam(rec_id,"localize-kappa",group,help7))
      paramList.append(RecipeParam(rec_id,"localize-niter",group,help8))
      paramList.append(RecipeParam(rec_id,"localize-use-skymask",group,help9))
      if rec_id == "xsh_scired_slit_nod" or rec_id == "xsh_respon_slit_nod":
         paramList.append(RecipeParam(rec_id,"localize-nod-throw",group,help10))

    def setOptExtractParameters(self,paramList,rec_id):
      group="optextract"
      help1="""TRUE if we do the optimal extraction."""
      help2="""Oversample factor for the science image."""
      help3="""Extraction box [pixel]."""
      help4="""Chunk size [bin]."""
      help5="""Lambda step [nm]."""
      help6="""Kappa for cosmics ray hits rejection."""
      help7="""Maximum bad pixels fraction for cosmics ray hits rejection."""
      help8="""Maximum number of iterations for cosmics ray hits rejection."""
      help9="""Number of iterations."""
      help10="""Extraction method GAUSSIAN | GENERAL."""


      paramList.append(RecipeParam(rec_id,"do-optextract",group,help1))
      paramList.append(RecipeParam(rec_id,"optextract-oversample",group,help2))
      paramList.append(RecipeParam(rec_id,"optextract-box-half-size",group,help3))
      paramList.append(RecipeParam(rec_id,"optextract-chunk-size",group,help4))
      paramList.append(RecipeParam(rec_id,"optextract-step-lambda",group,help5))
      paramList.append(RecipeParam(rec_id,"optextract-clip-kappa",group,help6))
      paramList.append(RecipeParam(rec_id,"optextract-clip-frac",group,help7))
      paramList.append(RecipeParam(rec_id,"optextract-clip-niter",group,help8))
      paramList.append(RecipeParam(rec_id,"optextract-niter",group,help9))
      paramList.append(RecipeParam(rec_id,"optextract-method",group,help10))

    def setRectifyParameters(self,paramList,rec_id):
      group="rectify"
      help1="""Name of the Interpolation Kernel Used. Possible values
               are tanh, tanh, sinc, sinc2, lanczos, hamming, hann."""
      help2="""Rectify Interpolation radius [bin units]."""
      help3="""Wavelength step in the output spectrum [nm]."""
      help4="""Spatial step along the slit in the output spectrum [arcsec]."""

      paramList.append(RecipeParam(rec_id,"rectify-kernel",group,help1))
      paramList.append(RecipeParam(rec_id,"rectify-radius",group,help2))
      paramList.append(RecipeParam(rec_id,"rectify-bin-lambda",group,help3))
      paramList.append(RecipeParam(rec_id,"rectify-bin-slit",group,help4))

    def setSkyParameters(self,paramList,rec_id):
      group="sky"
      help1="""TRUE to use subtract sky single."""
      help2="""Sky subtract Method (BSPLINE, MEDIAN)."""
      help3="""Nb of break points for Bezier curve fitting
                                (without localization)."""
      help4="""Nb of break points for Bezier curve fitting
                                 (with localization).""" 
      help5="""Bezier spline order."""
      help6="""Half size of the running median. If sky-method=MEDIAN"""
      help7="""Size of edges mask in arcsec."""
      help8="""Half size of the running median. If sky-method=MEDIAN."""
      help9="""Size of edges mask in arcsec."""
      help10="""Central position of the sky window #1 [arcsec]."""
      help11="""Half size of sky window #1 [arcsec]."""
      help12="""Central position of the sky window #2 [arcsec]."""
      help13="""Half size of the sky window #2 [arcsec]."""
        
      paramList.append(RecipeParam(rec_id,"sky-subtract",group,help1))
      paramList.append(RecipeParam(rec_id,"sky-method",group,help2))

      paramList.append(RecipeParam(rec_id,"sky-bspline-nbkpts-first",group,help3))
      paramList.append(RecipeParam(rec_id,"sky-bspline-nbkpts-second",group,help4))
      paramList.append(RecipeParam(rec_id,"sky-bspline-norder",group,help5))
      paramList.append(RecipeParam(rec_id,"sky-bspline-niter",group,help6))
      paramList.append(RecipeParam(rec_id,"sky-bspline-kappa",group,help7))
      paramList.append(RecipeParam(rec_id,"sky-median-hsize",group,help8))
      paramList.append(RecipeParam(rec_id,"sky-edges-mask",group,help9))
      paramList.append(RecipeParam(rec_id,"sky-position1",group,help10))
      paramList.append(RecipeParam(rec_id,"sky-hheight1",group,help11))
      paramList.append(RecipeParam(rec_id,"sky-position2",group,help12))
      paramList.append(RecipeParam(rec_id,"sky-hheight2",group,help13))


    def setCombineNodParameters(self,paramList,rec_id):
      group="combinenod"
      help1="""minimum NS size of resulting frame  in pixels."""
      help2="""TRUE if do sigma clipping. [TRUE]"""
      help3="""Kappa value in sigma clipping for combining nodded frames."""
      help4="""Number of iterations in sigma clipping for
                             combining nodded frames."""
      help5="""Minimal change in sigma in sigma clipping for
                          combining nodded frames."""
      help6="""Combination method for nodded frames (MEDIAN, MEAN)."""

      paramList.append(RecipeParam(rec_id,"combinenod-min",group,help1))
      paramList.append(RecipeParam(rec_id,"combinenod-clip",group,help2))
      paramList.append(RecipeParam(rec_id,"combinenod-clip-sigma",group,help3))
      paramList.append(RecipeParam(rec_id,"combinenod-clip-niter",group,help4))
      paramList.append(RecipeParam(rec_id,"combinenod-clip-diff",group,help5))
      paramList.append(RecipeParam(rec_id,"combinenod-method",group,help6))
	
    def setInteractiveParameters(self):
      paramList = list()

      rec_id=self.rec_id
      self.setGeneralParameters(paramList,rec_id)

      if rec_id[16:22] != "offset" :
         self.setStackParameters(paramList,rec_id)
      self.setCrhClipParameters(paramList,rec_id)
      self.setCrhSingleParameters(paramList,rec_id)

      self.setBackgroundParameters(paramList,rec_id)
      self.setLocalizeParameters(paramList,rec_id)
      self.setRectifyParameters(paramList,rec_id)

      if rec_id[16:21] == "stare" :
        self.setOptExtractParameters(paramList,rec_id)
        self.setSkyParameters(paramList,rec_id)

      if rec_id[17:19] == "nod" :
        self.setCombineNodParameters(paramList,rec_id)

      return paramList

    def setWindowHelp(self):
      help_text = """
This is an interactive window which help asses the quality of the execution of a recipe.
"""
      return help_text

    def setWindowTittle(self):
      tittle = 'X-shooter Interactive ' + self.obj_type + ' Reduction. ' + self.seq_arm + ' Arm. ' + self.ins_mode
      return tittle

except ImportError:
  import_sucess = 'false'
  print "Error importing modules pyfits, wx, matplotlib, numpy"
  raise

